@@ -75,6 +75,12 @@
 
 #define new PNEW
 
+extern "C" {
+  unsigned char linear2ulaw(int pcm_val);
+  int ulaw2linear(unsigned char u_val);
+};
+
+
 /* The class */
 GMManager::GMManager ()
 {
@@ -108,6 +114,7 @@ GMManager::GMManager ()
 #endif
 
   RTPTimer.SetNotifier (PCREATE_NOTIFIER (OnRTPTimeout));
+  AvgSignalTimer.SetNotifier (PCREATE_NOTIFIER (OnAvgSignalTimeout));
   GatewayIPTimer.SetNotifier (PCREATE_NOTIFIER (OnGatewayIPTimeout));
   GatewayIPTimer.RunContinuous (PTimeInterval (5));
 
@@ -619,8 +626,10 @@ GMManager::CreateSTUNClient (BOOL display_progress,
 {
   PWaitAndSignal m(sc_mutex);
 
-  if (sc)
+  if (sc) 
     delete (sc);
+
+  SetSTUNServer (PString ());
   
   /* Be a client for the specified STUN Server */
   sc = new GMStunClient (display_progress, 
@@ -639,6 +648,8 @@ GMManager::RemoveSTUNClient ()
   if (sc) 
     delete (sc);
   
+  SetSTUNServer (PString ());
+  
   sc = NULL;
 }
 
@@ -935,7 +946,8 @@ GMManager::OnEstablishedCall (OpalCall &call)
   gnomemeeting_threads_leave ();
   
   /* Update the timers */
-  RTPTimer.RunContinuous (PTimeInterval (0, 1));
+  RTPTimer.RunContinuous (PTimeInterval (1000));
+  AvgSignalTimer.RunContinuous (PTimeInterval (50));
 
   /* Update internal state */
   SetCallingState (GMManager::Connected);
@@ -1024,12 +1036,12 @@ GMManager::OnEstablished (OpalConnection &connection)
   if (audio_session) {
     
     audio_session->SetIgnoreOtherSources (TRUE);
-    audio_session->SetIgnorePayloadTypeChanges (FALSE);
+    audio_session->SetIgnorePayloadTypeChanges (TRUE);
   }
   if (video_session) {
 
     video_session->SetIgnoreOtherSources (TRUE);
-    video_session->SetIgnorePayloadTypeChanges (FALSE);
+    video_session->SetIgnorePayloadTypeChanges (TRUE);
   }
   
   if (!connection.IsOriginating ()) {
@@ -1076,6 +1088,7 @@ GMManager::OnClearedCall (OpalCall & call)
   if (GetCurrentCallToken() != PString::Empty() 
       && GetCurrentCallToken () != call.GetToken())
     return;
+
   
   /* Get the config settings */
   gnomemeeting_threads_enter ();
@@ -1091,10 +1104,12 @@ GMManager::OnClearedCall (OpalCall & call)
   
   /* we reset the no-data detection */
   RTPTimer.Stop ();
+  AvgSignalTimer.Stop ();
   stats.Reset ();
 
   /* Play busy tone */
-  pcssEP->PlaySoundEvent ("busy_tone_sound"); 
+  if (GetCallingState () == GMManager::Connected)
+    pcssEP->PlaySoundEvent ("busy_tone_sound"); 
 
   /* Update the various parts of the GUI */
   gnomemeeting_threads_enter ();
@@ -1118,12 +1133,18 @@ GMManager::OnClearedCall (OpalCall & call)
 					      GetCurrentCallToken (),
 					      GMManager::Standby);
 #endif
+  gm_main_window_push_message (main_window, 
+			       GetMissedCallsNumber (), 
+			       GetMWI ());
   gnomemeeting_threads_leave ();
 
   /* Update internal state */
   SetCallingState (GMManager::Standby);
   SetCurrentCallToken ("");
 
+  /* Reinitialize codecs */
+  re_audio_codec = tr_audio_codec = re_video_codec = tr_video_codec = "";
+
   /* Try to update the devices use if some settings were changed 
      during the call */
   UpdateDevices ();
@@ -1158,10 +1179,6 @@ GMManager::OnReleased (OpalConnection & connection)
     return;
   }
 
-  /* we reset the no-data detection */
-  RTPTimer.Stop ();
-  stats.Reset ();
-  
   /* Start time */
   if (connection.GetConnectionStartTime ().IsValid ())
     t = PTime () - connection.GetConnectionStartTime();
@@ -1273,9 +1290,6 @@ GMManager::OnReleased (OpalConnection & connection)
 				 msg_reason,
 				 utf8_app);
   gm_history_window_insert (history_window, msg_reason);
-  gm_main_window_push_message (main_window, 
-			       GetMissedCallsNumber (), 
-			       GetMWI ());
   gm_main_window_flash_message (main_window, msg_reason);
   gm_chat_window_push_info_message (chat_window, NULL, "");
   gnomemeeting_threads_leave ();
@@ -1285,9 +1299,6 @@ GMManager::OnReleased (OpalConnection & connection)
   g_free (utf8_url);  
   g_free (msg_reason);
 
-  /* Reinitialize codecs */
-  re_audio_codec = tr_audio_codec = re_video_codec = tr_video_codec = "";
-
   PTRACE (3, "GMManager\t Will release the connection");
   OpalManager::OnReleased (connection);
 }
@@ -1981,6 +1992,51 @@ GMManager::UpdateRTPStats (PTime start_time,
 }
 
 
+void 
+GMManager::OnAvgSignalTimeout (PTimer &,
+                               INT)
+{
+  GtkWidget *main_window = NULL;
+  
+  PSafePtr <OpalCall> call = NULL;
+  PSafePtr <OpalConnection> connection = NULL;
+
+  OpalRawMediaStream *audio_stream = NULL;
+
+  float output = 0;
+  float input = 0;
+  
+  main_window = GnomeMeeting::Process ()->GetMainWindow ();
+
+  call = FindCallWithLock (GetCurrentCallToken ());
+  if (call != NULL) {
+
+    connection = GetConnection (call, FALSE);
+
+    if (connection != NULL) {
+
+      audio_stream = 
+        (OpalRawMediaStream *) 
+        connection->GetMediaStream (OpalMediaFormat::DefaultAudioSessionID, 
+                                    FALSE);
+      if (audio_stream)
+        output = (linear2ulaw (audio_stream->GetAverageSignalLevel ()) ^ 0xff) / 100.0;
+
+      audio_stream = 
+        (OpalRawMediaStream *) 
+        connection->GetMediaStream (OpalMediaFormat::DefaultAudioSessionID, 
+                                    TRUE);
+      if (audio_stream)
+        input = (linear2ulaw (audio_stream->GetAverageSignalLevel ()) ^ 0xff) / 100.0;
+    } 
+
+    gdk_threads_enter ();
+    gm_main_window_set_signal_levels (main_window, output, input);
+    gdk_threads_leave ();
+  }
+}
+
+
 void 
 GMManager::OnRTPTimeout (PTimer &, 
 			  INT)
